#! /usr/bin/env python
import os
from pathlib import Path


DEBUG = True
SECRET_KEY = os.environ.get("SECRET_KEY", os.urandom(32))
ALLOWED_HOSTS = os.environ.get("ALLOWED_HOSTS", "localhost,127.0.0.1").split(",")

BASE_DIR = Path(__file__).resolve().parent.parent
VAR_DIR = BASE_DIR.joinpath("var")
VAR_DIR.mkdir(exist_ok=True, parents=True)
STATIC_ROOT = VAR_DIR.joinpath("static")
STATIC_ROOT.mkdir(exist_ok=True, parents=True)

SITE_ID = 1
ROOT_URLCONF = "tests.project"
MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    # "django.contrib.sites.middleware.CurrentSiteMiddleware",
]
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": VAR_DIR / "db.sqlite3",
    },
}
INSTALLED_APPS = [
    "django_extensions",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.staticfiles",
    # "django.contrib.sites",
    "sitevars",
]
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {"console": {"class": "logging.StreamHandler"}},
    "loggers": {"": {"handlers": ["console"], "level": "INFO"}},
}
MESSAGE_STORAGE = "django.contrib.messages.storage.cookie.CookieStorage"
SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"
SESSION_COOKIE_HTTPONLY = True
STATIC_URL = "static/"
STORAGES = {
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
        "LOCATION": VAR_DIR / "media",
    },
    "staticfiles": {
        "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
        "LOCATION": STATIC_ROOT,
    },
}
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.messages.context_processors.messages",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
            ]
        },
    }
]
USE_TZ = True
WSGI_APPLICATION = "tests.project.application"



#!/usr/bin/env python
import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner

if __name__ == "__main__":
    settings_files = {
        "contrib_sites": "tests.contrib_sites",
        "no_sites": "tests.no_sites",
        "alt_sites": "tests.alt_sites",
    }
    if len(sys.argv) > 1 and sys.argv[1] in settings_files:
        settings_module = settings_files[sys.argv[1]]
    else:
        # Prompt the user to select a settings file
        print("Please select a settings file:")
        for key in settings_files:
            print(f"  {key}")
        settings_module = settings_files[input().strip()]

    os.environ["DJANGO_SETTINGS_MODULE"] = settings_module
    django.setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(["tests"])
    sys.exit(bool(failures))



import typing as T

from django.apps import apps
from django.core.cache import cache
from django.db import models, transaction
from django.utils.translation import gettext_lazy as _

config = apps.get_app_config("sitevars")


class SiteVarQueryset(models.QuerySet):
    def get_value(self, name: str, default: str = "", asa: T.Callable = str):
        """
        Given a queryset pre-filtered by site, returns the value of the SiteVar with
        the given name. If no value is set for that name, returns the passed default
        value, or empty string if no default was passed. To transform the stored string
        to another type, pass a transform function in the asa argument. The default
        value should be passed as a string; it will be passed to the asa function
        for transformation.

        Examples:

        # Returns the string if set, or "" if not set
        x = site.vars.get_value("analytics_id")
        # Returns the string if set, or "Ignore" if not set
        x = site.vars.get_value("abort_retry_ignore", "Ignore")
        # Returns the number of pages as an integer. Note the default should be a str.
        num_items = site.vars.get_value("paginate_by", default="10", asa=int)
        # Parses the value as JSON and returns the result
        data = site.vars.get_value("json_data", "{}", json.loads)
        """
        conf = apps.get_app_config("sitevars")
        site_id: int = 0
        # If using the PlaceholderSite model, use the hardcoded ID
        if config.site_model == "sitevars.PlaceholderSite":
            site_id = 1

        # Otherwise, determine the site ID from the queryset
        if not site_id:
            for lookup in self.query.where.children:
                if not isinstance(lookup, models.fields.related_lookups.RelatedExact):
                    continue
                if lookup.lhs.target.name == "site":
                    site_id = lookup.rhs
                    break

        if not site_id:
            raise ValueError("get_value requires a queryset filtered by site")

        # Check whether we are operating inside a transaction
        in_transaction = not transaction.get_connection().get_autocommit()

        # It's not safe to use the cache in a transaction, as it can get out of sync
        if conf.use_cache and not in_transaction:
            # Construct the cache key and retrieve the cached value
            key = f"sitevars:{site_id}"
            allvars = cache.get(key, None)
            if allvars is None:
                # Cache is empty, populate the cache
                allvars = {var.name: var.value for var in self.all()}
                cache.set(key, allvars)
            val = allvars.get(name, default)
            return asa(val) if val is not None else val

        # Not using cache, just query the DB
        try:
            return asa(self.get(name=name).value)
        except self.model.DoesNotExist:
            # This allows None as a default, without crashing on e.g. `int(None)`
            return asa(default) if default is not None else default
        # Note explicitly NOT catching MultipleObjectsReturned, that's still an error

    def clear_cache(self, site_id: T.Optional[int] = None):
        """
        Clear the cache for the given site_id, or all sites if no site_id is given.
        """
        Site = apps.get_model(*config.site_model.split("."))
        if site_id is not None:
            key = f"sitevars:{site_id}"
            cache.delete(key)
        else:
            for site_id in Site.objects.values_list("pk", flat=True):
                key = f"sitevars:{site_id}"
                cache.delete(key)


class SiteVar(models.Model):
    """
    Site-specific variables are stored here. All site variable are injected into
    template contexts using the context processor in
    ``sitevars.context_processors.inject_sitevars``.
    """

    site = models.ForeignKey(
        config.site_model,
        verbose_name=_("site"),
        on_delete=models.CASCADE,
        related_name="vars",
    )
    name = models.CharField(_("name"), max_length=100)
    value = models.TextField(_("value"))

    objects = SiteVarQueryset.as_manager()

    class Meta:
        base_manager_name = "objects"
        unique_together = ("site", "name")
        verbose_name = _("site variable")
        verbose_name_plural = _("site variables")

    def __str__(self):
        return f"{self.name}={self.value} ({self.site.domain})"

    def save(self, *args, **kwargs):
        # Clear the cache if it exists
        transaction.on_commit(lambda: self.__class__.objects.clear_cache(self.site.id))
        return super().save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        # Clear the cache if it exists
        transaction.on_commit(lambda: self.__class__.objects.clear_cache(self.site.id))
        return super().delete(*args, **kwargs)


class PlaceholderSite(models.Model):
    """
    A placeholder site model to use when the Django's Site model is not available.
    """

    domain = models.CharField(_("domain name"), max_length=100, blank=True)
    name = models.CharField(_("display name"), max_length=50, blank=True)

    def __str__(self):
        return "Placeholder Site"



from django.contrib import admin
from django.core.wsgi import get_wsgi_application
from django.urls import path

urlpatterns = [
    path("admin/", admin.site.urls),
]

application = get_wsgi_application()



from django.core.checks import Tags, Warning, register


@register(Tags.admin)
def check_contrib_sites_comes_before_sitevars(app_configs, **kwargs):
    """
    Checks that contrib.sites is installed before sitevars. If it's not, the admin
    registration will fail and the SiteAdmin will not have the sitevars inlines.
    """
    from django.conf import settings

    if (
        "sitevars" in settings.INSTALLED_APPS
        and "django.contrib.sites" in settings.INSTALLED_APPS
    ):
        sitevars_index = settings.INSTALLED_APPS.index("sitevars")
        sites_index = settings.INSTALLED_APPS.index("django.contrib.sites")
        if sitevars_index < sites_index:
            return [
                Warning(
                    "sitevars is installed before django.contrib.sites",
                    hint=(
                        "contrib.sites must be installed before sitevars in INSTALLED_APPS "
                        "to ensure SiteVars are editable in the SiteAdmin."
                    ),
                    id="sitevars.W001",
                    obj="sitevars",
                )
            ]
    return []


@register(Tags.sites)
def check_request_context_processor_is_installed(app_configs, **kwargs):
    """
    Checks that the ``request`` context processor is installed, because the sitevar
    template tag requires it.
    """
    from django.conf import settings

    using_context_processor = False
    for backend in settings.TEMPLATES:
        if "django.template.context_processors.request" in backend.get(
            "OPTIONS", {}
        ).get("context_processors", []):
            using_context_processor = True
            break

    if not using_context_processor:
        return [
            Warning(
                "'django.template.context_processors.request' must be enabled in "
                "DjangoTemplates (TEMPLATES) in order to use the sitevar template tag.",
                id="sitevars.W002",
                obj="sitevars",
            )
        ]
    return []



"""
Test sitevars with django.contrib.sites installed.
"""

from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent
# VAR_DIR = BASE_DIR.joinpath("var")
# VAR_DIR.mkdir(exist_ok=True, parents=True)
# STATIC_ROOT = VAR_DIR.joinpath("static")
# STATIC_ROOT.mkdir(exist_ok=True, parents=True)
STATIC_URL = "/static/"
SITE_ID = 1
SECRET_KEY = "INSECURE! For testing only."
DEBUG = False
ALLOWED_HOSTS = ["*"]
ROOT_URLCONF = "tests.project"
WSGI_APPLICATION = "tests.project.application"
MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.sites.middleware.CurrentSiteMiddleware",
]
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "contrib_sites.sqlite3",
    },
}
INSTALLED_APPS = [
    "django_extensions",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "sitevars",
]
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.messages.context_processors.messages",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
            ]
        },
    }
]



from django.apps import apps
from django.core.cache import cache


# A context processor to add our vars to template contexts:
def inject_sitevars(request):
    """Add all sitevars to the template context."""
    conf = apps.get_app_config("sitevars")
    SiteVar = conf.get_model("SiteVar")

    # Get the site_id, or raise ImproperlConfigured
    site_id = conf.get_site_id_for_request(request)

    qs = SiteVar.objects.filter(site_id=site_id)

    if not conf.use_cache:
        return {var.name: var.value for var in qs}

    # Construct the cache key and retrieve the cached value
    key = f"sitevars:{site_id}"
    allvars = cache.get(key, None)
    if allvars is None:
        # Empty cache, populate the cache
        allvars = {var.name: var.value for var in qs}
        cache.set(key, allvars)
    return allvars



import logging
from django.apps import AppConfig
from django.apps import apps as global_apps
from django.db import DEFAULT_DB_ALIAS, router
from django.db.models.signals import post_migrate

logger = logging.getLogger(__name__)


def create_fake_site(
    app_config,
    verbosity=2,
    interactive=True,
    using=DEFAULT_DB_ALIAS,
    apps=global_apps,
    **kwargs,
):
    """
    Create the default FakeSite object.
    """
    try:
        Site = apps.get_model("tests", "FakeSite")
    except LookupError:
        return

    if not router.allow_migrate_model(using, Site):
        return

    if not Site.objects.using(using).exists():
        Site(pk=1, domain="example.com", name="example.com").save(using=using)


class FakeSiteConfig(AppConfig):
    name = "tests"
    verbose_name = "Fake Site"

    def ready(self):
        post_migrate.connect(create_fake_site, sender=self)



from django.apps import AppConfig
from django.apps import apps as global_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import DEFAULT_DB_ALIAS, router
from django.db.models.signals import post_migrate
from django.utils.module_loading import import_string

# Import the checks module to register system checks
import sitevars.checks  # noqa: F401


def create_default_site(
    app_config,
    verbosity=2,
    interactive=True,
    using=DEFAULT_DB_ALIAS,
    apps=global_apps,
    **kwargs,
):
    """
    Create the default singleton PlaceholderSite object.
    """
    # If the model doesn't exist, this is a legacy installation using
    # django.contrib.sites upgraded from 1.x. Nothing to do, we won't need the
    # PlaceholderSite.
    try:
        Site = apps.get_model("sitevars", "PlaceholderSite")
    except LookupError:
        return

    if not router.allow_migrate_model(using, Site):
        return

    # If the table is empty, create the default site
    if not Site.objects.using(using).exists():
        Site(pk=1, domain="example.com", name="example.com").save(using=using)


class SitevarsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "sitevars"

    @property
    def use_cache(self):
        return getattr(settings, "SITEVARS_USE_CACHE", True)

    @property
    def site_model(self):
        """
        Return the name of the Site model to use for foreign keys.
        """
        name = getattr(settings, "SITE_MODEL", None)
        if not name and "django.contrib.sites" in settings.INSTALLED_APPS:
            name = "sites.Site"
        if not name:
            name = "sitevars.PlaceholderSite"
        return name

    def ready(self):
        post_migrate.connect(create_default_site, sender=self)

    def get_site_id_for_request(self, request=None):
        """
        Return the site_id for the current request.
        """
        # Shortcut if we're using our PlaceholderSite model
        if self.site_model == "sitevars.PlaceholderSite":
            return 1

        # To save queries, cache site_id on the request object
        if request and hasattr(request, "_sitevars_site_id"):
            return request._sitevars_site_id

        # Use the site middleware if available
        if request and hasattr(request, "site") and hasattr(request.site, "id"):
            request._sitevars_site_id = request.site.id
            return request.site.id

        # If a function is configured, use that to get the site_id
        if hasattr(settings, "CURRENT_SITE_FUNCTION"):
            func = import_string(settings.CURRENT_SITE_FUNCTION)
            site_id = func(request).id
            request._sitevars_site_id = site_id
            return site_id

        Site = global_apps.get_model(*self.site_model.split("."))
        # If a method is configured, use that to get the site_id
        if hasattr(settings, "CURRENT_SITE_METHOD"):
            site_id = getattr(Site, settings.CURRENT_SITE_METHOD)(request).id
            request._sitevars_site_id = site_id
            return site_id

        # Fallback to the sites framework's get_current() method
        if hasattr(Site.objects, "get_current"):
            site_id = Site.objects.get_current(request).id
            request._sitevars_site_id = site_id
            return site_id

        # exhausted all possibilities
        host = request.get_host() if request else "Unknown domain"
        msg = (
            "Unable to identify a site for this request. "
            "Check your settings for SITE_MODEL, CURRENT_SITE_FUNCTION, or "
            "CURRENT_SITE_METHOD configuration."
        )
        raise ImproperlyConfigured(
            msg,
            request,
            host,
        )



import logging

from django.apps import apps
from django.conf import settings
from django.contrib import admin
from django import forms

from sitevars.models import SiteVar, PlaceholderSite

logger = logging.getLogger("sitevars")


# These admin classes are for use with django.contrib.sites.
class SiteVarInline(admin.TabularInline):
    extra: int = 1
    model = SiteVar


class ContribSiteAdmin(admin.ModelAdmin):
    list_display = ("domain", "name")
    search_fields = ("domain", "name")
    inlines = [SiteVarInline]


# This class should be usable with any sites framework.
class SiteVarAdmin(admin.ModelAdmin):
    list_display = ("name", "value", "site")
    list_filter = ("site",)
    list_editable = ("value",)
    search_fields = ("name", "value")
    ordering = ("site", "name")


# This form and admin class are used only when using the PlaceholderSite model.
class PlaceholderSiteVarAdminForm(forms.ModelForm):
    class Meta:
        model = SiteVar
        fields = ["site", "name", "value"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Check if we're using the PlaceholderSite model
        model = apps.get_app_config("sitevars").site_model
        if model.lower() == "sitevars.placeholdersite":
            # Hide the site field and set default to PlaceholderSite with id=1
            self.fields["site"].widget = forms.HiddenInput()
            # Should have been created at migration time, but JIC
            self.initial["site"] = PlaceholderSite.objects.get_or_create(id=1)[0]


class PlaceholderSiteVarAdmin(admin.ModelAdmin):
    form = PlaceholderSiteVarAdminForm
    list_display = ("name", "value")
    list_editable = ("value",)
    search_fields = ("name", "value")
    ordering = ("name",)


if "django.contrib.sites" in settings.INSTALLED_APPS:
    # Note: if this raises admin.NotRegistered, that means contrib.sites is not installed,
    # or is installed after sitevars. Trying to register our SiteAdmin would cause a crash.
    try:
        # Unregister the default SiteAdmin and register our ContribSiteAdmin
        from django.contrib.sites.models import Site

        admin.sites.site.unregister(Site)
        admin.site.register(Site, ContribSiteAdmin)
        admin.site.register(SiteVar, SiteVarAdmin)

    except admin.exceptions.NotRegistered:
        logger.warning(
            "SiteAdmin was not registered. "
            "Place sitevars AFTER contrib.sites in INSTALLED_APPS."
        )
        pass

elif apps.get_app_config("sitevars").site_model.lower() == "sitevars.placeholdersite":
    # If we're using the PlaceholderSite model, register the PlaceholderSiteVarAdmin
    admin.site.register(SiteVar, PlaceholderSiteVarAdmin)

else:
    admin.site.register(SiteVar, SiteVarAdmin)



# Generated by Django 5.1.7 on 2025-03-16 12:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FakeSite',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=100)),
                ('domain', models.CharField(blank=True, max_length=100)),
            ],
        ),
    ]



from django import template
from django.apps import apps

register = template.Library()
config = apps.get_app_config("sitevars")


@register.simple_tag(takes_context=True)
def sitevar(context, var_name, default=""):
    """
    Inserts the value of a site variable for the current site.

    Usage::

        {% load sitevars %}
        {% sitevar "my_var" %}
        {% sitevar "my_var" "default" %}

    Note: If you are using the ``sitevars.context_processors.inject_sitevars``
    context processor, you can access site variables directly in templates. ::

        {{ my_var|default:"default" }}

    """
    SiteVar = config.get_model("SiteVar")

    # Shortcut when using PlaceholderSite, don't even need the request
    if config.site_model.lower() == "sitevars.placeholdersite":
        return SiteVar.objects.get_value(var_name, default)

    # Get the site_id for the current site
    site_id = config.get_site_id_for_request(context["request"])

    return SiteVar.objects.filter(site_id=site_id).get_value(var_name, default)



# Generated by Django 5.0.7 on 2025-03-15 13:08

import django.db.models.deletion
from django.apps import apps
from django.db import migrations, models


config = apps.get_app_config("sitevars")


class Migration(migrations.Migration):
    # HACK: If the project has configured a custom site model (or has django.contrib.sites
    # installed), add a migration dependency on it. If not, no dependency is needed, we'll
    # use our PlaceholderSite model.
    dependencies = (
        []
        if config.site_model.lower() == "sitevars.placeholdersite"
        else [migrations.swappable_dependency(config.site_model)]
    )

    operations = [
        migrations.CreateModel(
            name="PlaceholderSite",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "domain",
                    models.CharField(
                        max_length=100, verbose_name="domain name", blank=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        max_length=50, verbose_name="display name", blank=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SiteVar",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
                ("value", models.TextField(verbose_name="value")),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="vars",
                        to=config.site_model,
                        verbose_name="site",
                    ),
                ),
            ],
            options={
                "verbose_name": "site variable",
                "verbose_name_plural": "site variables",
                "base_manager_name": "objects",
                "unique_together": {("site", "name")},
            },
        ),
    ]



from unittest import skipIf
from unittest.mock import Mock, patch, ANY

from django.apps import apps
from django.contrib.admin import site as adminsite
from django.contrib.auth.models import User
from django.core.checks import Warning
from django.db import transaction
from django.db.utils import IntegrityError
from django.template import Context, Template
from django.test import TestCase, TransactionTestCase, override_settings, RequestFactory
from django.urls import reverse

from sitevars import checks
from sitevars.context_processors import inject_sitevars
from sitevars.models import SiteVar

config = apps.get_app_config("sitevars")


class AdminSmokeTest(TestCase):
    @classmethod
    def setUpTestData(cls) -> None:
        cls.user = User.objects.create(
            username="test_admin",
            password="super-secure",
            is_staff=True,
            is_superuser=True,
        )
        return super().setUpTestData()

    def test_load_admin_pages(self):
        """Load each admin change and add page to check syntax in the admin classes."""
        self.client.force_login(self.user)

        app_label = "sitevars"
        app = apps.get_app_config(app_label)
        for model in app.get_models():
            if not adminsite.is_registered(model):
                continue

            with self.subTest(model=model):
                changelist_url = reverse(
                    f"admin:{app_label}_{model._meta.model_name}_changelist"
                )
                add_url = reverse(f"admin:{app_label}_{model._meta.model_name}_add")
                resp_changelist = self.client.get(changelist_url)
                resp_add = self.client.get(add_url)
                self.assertEqual(resp_changelist.status_code, 200)
                self.assertEqual(resp_add.status_code, 200)


class AppConfigTest(TestCase):
    @override_settings(SITEVARS_USE_CACHE=False)
    def test_use_cache_false(self):
        """Test the use_cache property when SITEVARS_USE_CACHE is False."""
        self.assertFalse(config.use_cache)

    def test_use_cache_default(self):
        """Test the use_cache property."""
        self.assertTrue(config.use_cache)

    @skipIf(
        config.site_model != "sitevars.PlaceholderSite",
        "Test only applies to PlaceholderSite model.",
    )
    def test_get_site_id_for_request__placeholder_site(self):
        """Test the get_site_id_for_request method."""
        request = RequestFactory().get("/")
        self.assertEqual(config.get_site_id_for_request(request), 1)

    @skipIf(
        config.site_model == "sitevars.PlaceholderSite",
        "Not used with PlaceholderSite model.",
    )
    def test_get_site_id_for_request__site_middleware(self):
        """Test the get_site_id_for_request when request.site is valid."""
        request = RequestFactory().get("/")
        request.site = Mock()
        request.site.id = 7
        self.assertEqual(config.get_site_id_for_request(request), 7)

    @skipIf(
        config.site_model != "tests.FakeSite",
        "Only applies to custom SITE_MODEL.",
    )
    def test_get_site_id_for_request__current_site_function(self):
        """Test the get_site_id_for_request when CURRENT_SITE_FUNCTION is set."""
        with override_settings(CURRENT_SITE_FUNCTION="tests.models.get_current_site"):
            request = RequestFactory().get("/")
            with self.assertLogs("sitevars.testing", "INFO") as cm:
                self.assertEqual(config.get_site_id_for_request(request), 1)
            self.assertIn("INFO:sitevars.testing:get_current_site() called", cm.output)

    @skipIf(
        config.site_model != "tests.FakeSite",
        "Only applies to custom SITE_MODEL.",
    )
    def test_get_site_id_for_request__current_site_method(self):
        """Test the get_site_id_for_request when CURRENT_SITE_METHOD is set."""
        with override_settings(CURRENT_SITE_METHOD="get_current"):
            request = RequestFactory().get("/")
            with self.assertLogs("sitevars.testing", "INFO") as cm:
                self.assertEqual(config.get_site_id_for_request(request), 1)
            self.assertIn(
                "INFO:sitevars.testing:FakeSite.get_current() called", cm.output
            )

    @skipIf(
        config.site_model != "tests.FakeSite",
        "Only applies to contrib.sites or compatible.",
    )
    def test_get_site_id_for_request__fallback_to_get_current(self):
        """Test that it falls back to Site.objects.get_current"""
        request = RequestFactory().get("/")
        with self.assertLogs("sitevars.testing", "INFO") as cm:
            self.assertEqual(config.get_site_id_for_request(request), 1)
        self.assertEqual(
            cm.output, ["INFO:sitevars.testing:FakeSiteManager.get_current() called"]
        )


class ContextProcessorTest(TestCase):
    def test_context_processor_returns_dict_with_one_query(self):
        """Test the context processor "happy path"."""
        # Create a sitevar
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")

        # Test the context processor returns the sitevar and populates the cache
        with patch("sitevars.context_processors.cache") as mock_cache:
            request = RequestFactory().get("/")
            # Simulate site middleware
            request.site = Mock()
            request.site.id = 1
            mock_cache.get.return_value = None
            with self.assertNumQueries(1):
                context = inject_sitevars(request)
            self.assertEqual(context, {"testvar": "testvalue"})
            mock_cache.get.assert_called_once_with("sitevars:1", None)
            mock_cache.set.assert_called_once_with(
                "sitevars:1", {"testvar": "testvalue"}
            )

    def test_context_processor_returns_dict__without_site_middleware(self):
        """Test the context processor when sites middleware not installed."""
        # Create a sitevar
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")

        # Test the context processor returns the sitevar and populates the cache
        with patch("sitevars.context_processors.cache") as mock_cache:
            request = RequestFactory().get("/")
            assert not hasattr(request, "site")
            mock_cache.get.return_value = None

            context = inject_sitevars(request)

            self.assertEqual(context, {"testvar": "testvalue"})

    def test_cache_used(self):
        """Test that the context processor uses the cache."""
        # Create a sitevar
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")

        with patch("sitevars.context_processors.cache") as mock_cache:
            request = RequestFactory().get("/")
            request.site = Mock()
            request.site.id = 1
            mock_cache.get.return_value = {"testvar": "testvalue"}
            with self.assertNumQueries(0):
                context = inject_sitevars(request)
            self.assertEqual(context, {"testvar": "testvalue"})
            mock_cache.get.assert_called_once_with("sitevars:1", None)
            mock_cache.set.assert_not_called()

    @override_settings(SITEVARS_USE_CACHE=False)
    def test_context_processor_caching_off(self):
        """Test the context processor with caching off."""
        conf = apps.get_app_config("sitevars")
        self.assertFalse(conf.use_cache)  # Because SITEVARS_USE_CACHE=False

        # Create a sitevar
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")

        with patch("sitevars.context_processors.cache") as mock_cache:
            request = RequestFactory().get("/")
            request.site = Mock()
            request.site.id = 1
            context = inject_sitevars(request)
            self.assertEqual(context, {"testvar": "testvalue"})
            mock_cache.get.assert_not_called()
            mock_cache.set.assert_not_called()
        self.assertEqual(context, {"testvar": "testvalue"})


class SiteVarModelTest(TransactionTestCase):
    # Note: we use TransactionTestCase to manually manage transactions where TestCase
    # would rollback the transaction before the cache is cleared.
    def test_sitevar_str(self):
        """Test the string representation of a sitevar."""
        sitevar = SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        self.assertEqual(str(sitevar), "testvar=testvalue (example.com)")

    def test_sitevar_unique_together(self):
        """Test that sitevar names are unique per site."""
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        with self.assertRaises(IntegrityError):
            SiteVar.objects.create(site_id=1, name="testvar", value="othervalue")

    @skipIf(
        config.site_model.lower() == "sitevars.placeholdersite",
        "Test does not apply to when using PlaceholderSite model.",
    )
    def test_sitevar_unique_together_different_sites(self):
        """Test that sitevar names are not unique across different sites."""
        Site = apps.get_model(*config.site_model.split("."))
        site1 = Site.objects.get(pk=1)
        site2 = Site.objects.create(domain="example2.com", name="example2.com")
        SiteVar.objects.create(site=site1, name="testvar", value="testvalue")
        SiteVar.objects.create(site=site2, name="testvar", value="othervalue")
        self.assertEqual(
            SiteVar.objects.filter(site=site2).get_value("testvar"), "othervalue"
        )
        self.assertEqual(
            SiteVar.objects.filter(site=site1).get_value("testvar"), "testvalue"
        )

    @skipIf(
        config.site_model.lower() == "sitevars.placeholdersite",
        "Test does not apply to when using PlaceholderSite model.",
    )
    def test_get_value_requires_queryset_filtered_by_site(self):
        """Test that get_value raises an error when the queryset is not filtered by site."""
        with self.assertRaises(ValueError):
            SiteVar.objects.get_value("testvar")

    @skipIf(
        config.site_model.lower() != "sitevars.placeholdersite",
        "Test only applies to PlaceholderSite model.",
    )
    def test_get_value_placeholder_site(self):
        """Test that get_value automatically filters queries when using the placeholder site."""
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        with self.assertNumQueries(1):
            self.assertEqual(SiteVar.objects.get_value("testvar"), "testvalue")

    @override_settings(SITEVARS_USE_CACHE=False)
    def test_sitevar_get_value_no_cache(self):
        """Test that get_value honors the use_cache app setting."""
        Site = apps.get_model(*config.site_model.split("."))
        site = Site.objects.get(pk=1)
        with patch("sitevars.models.cache") as mock_cache:
            mock_cache.get.return_value = None
            SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
            self.assertEqual(site.vars.get_value("testvar"), "testvalue")
            mock_cache.get.assert_not_called()
            mock_cache.set.assert_not_called()
            with self.assertNumQueries(1):
                self.assertEqual(
                    site.vars.get_value("nonexistent", None, asa=int),
                    None,
                )

    def test_sitevar_get_value_cache_hit(self):
        """Test that get_value uses the cache."""
        Site = apps.get_model(*config.site_model.split("."))
        site = Site.objects.get(pk=1)
        with patch("sitevars.models.cache") as mock_cache:
            mock_cache.get.return_value = {"testvar": "testvalue"}
            with self.assertNumQueries(0):
                self.assertEqual(site.vars.get_value("testvar"), "testvalue")
            mock_cache.get.assert_called_once_with("sitevars:1", None)
            mock_cache.set.assert_not_called()

    def test_get_value_ignores_cache_inside_transaction(self):
        """Test that the cache is ignored inside a transaction.

        Because transactions can rollback (and there is no transaction.on_rollback we
        can use to detect that), there's a potential for the cache to get out of sync
        if we both write (which clears the cache) and then read (which repopulates the
        cache) inside a transaction that is then rolled back. To avoid this, we ignore
        the cache inside a transaction. This is an edge case that is unlikely (though
        not impossible) to occur in production use, but always happens in TestCase
        tests (which is why we use TransactionTestCase).
        """
        Site = apps.get_model(*config.site_model.split("."))
        site = Site.objects.get(pk=1)
        with transaction.atomic():
            # Attempt to retrieve a sitevar. This would normally populate the cache.
            with patch("sitevars.models.cache") as mock_cache:
                val = site.vars.get_value("testvar", None)
                mock_cache.get.assert_not_called()
                mock_cache.set.assert_not_called()
                self.assertIsNone(val)

            # Cache should still be cleared on write
            with patch("sitevars.models.transaction") as mock_xact:
                SiteVar.objects.create(site=site, name="testvar", value="testvalue")
                mock_xact.on_commit.assert_called_once_with(ANY)

            # The cache should not be used here
            with patch("sitevars.models.cache") as mock_cache:
                with self.assertNumQueries(1):
                    self.assertEqual(site.vars.get_value("testvar"), "testvalue")
                mock_cache.get.assert_not_called()
                mock_cache.set.assert_not_called()

    def test_sitevar_clear_cache(self):
        """Test that the cache is cleared correctly."""
        SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        with patch("sitevars.models.cache") as mock_cache:
            SiteVar.objects.clear_cache(site_id=1)
            mock_cache.delete.assert_called_once_with("sitevars:1")

    def test_sitevar_clear_cache_all_sites(self):
        """Test that the cache is cleared for all sites."""
        Site = apps.get_model(*config.site_model.split("."))
        site1 = Site.objects.get(pk=1)
        site2 = Site.objects.create(domain="example2.com", name="example2.com")
        SiteVar.objects.create(site=site1, name="testvar", value="testvalue")
        SiteVar.objects.create(site=site2, name="testvar", value="othervalue")
        with patch("sitevars.models.cache") as mock_cache:
            SiteVar.objects.clear_cache()
            mock_cache.delete.assert_any_call("sitevars:1")
            mock_cache.delete.assert_any_call("sitevars:2")

    def test_delete_clears_cache(self):
        """Test that the cache is cleared on commit when a sitevar is deleted."""
        sitevar = SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        with patch("sitevars.models.transaction") as mock_xact:
            sitevar.delete()
            mock_xact.on_commit.assert_called()

    def test_save_clears_cache(self):
        """Test that the cache is cleared on commit when a sitevar is saved."""
        sitevar = SiteVar.objects.create(site_id=1, name="testvar", value="testvalue")
        with patch("sitevars.models.transaction") as mock_xact:
            sitevar.save()
            mock_xact.on_commit.assert_called()


class SiteVarTemplateTagTest(TestCase):
    @classmethod
    def setUpTestData(cls) -> None:
        Site = apps.get_model(*config.site_model.split("."))
        cls.site = Site.objects.get(pk=1)
        cls.sitevar = SiteVar.objects.create(
            site=cls.site, name="testvar", value="testvalue"
        )
        cls.request = RequestFactory().get("/")
        cls.request.site = cls.site

    def test_sitevar_exists(self):
        """Test that the sitevar is retrieved correctly."""
        template = Template("{% load sitevars %}{% sitevar 'testvar' %}")
        rendered = template.render(Context({"request": self.request}))
        self.assertEqual(rendered.strip(), "testvalue")

    def test_sitevar_not_found(self):
        """Test that the default value is returned when sitevar is not found."""
        template = Template(
            "{% load sitevars %}{% sitevar 'nonexistent' 'defaultvalue' %}"
        )
        rendered = template.render(Context({"request": self.request}))
        self.assertEqual(rendered.strip(), "defaultvalue")

    def test_sitevar_with_context(self):
        """Test that the sitevar is retrieved correctly with context."""
        template = Template(
            "{% load sitevars %}{% sitevar 'testvar' as var %}{{ var }}"
        )
        rendered = template.render(Context({"request": self.request}))
        self.assertEqual(rendered.strip(), "testvalue")

    def test_sitevar_with_context_and_default(self):
        """Test that the sitevar is retrieved correctly with context and default value."""
        template = Template(
            "{% load sitevars %}{% sitevar 'nonexistent' default='defaultvalue' as var %}{{ var }}"
        )
        rendered = template.render(Context({"request": self.request}))
        self.assertEqual(rendered.strip(), "defaultvalue")

    def test_sitevar_no_site_middleware(self):
        """Test that the sitevar is retrieved correctly without site middleware."""
        template = Template("{% load sitevars %}{% sitevar 'testvar' %}")
        request = RequestFactory().get("/")
        assert not hasattr(request, "site")

        rendered = template.render(Context({"request": request}))
        self.assertEqual(rendered.strip(), "testvalue")

    @skipIf(
        config.site_model.lower() != "sitevars.placeholdersite",
        "Test only applies to PlaceholderSite model.",
    )
    def test_sitevar_placeholder_without_request_context(self):
        """Test that the sitevar is retrieved correctly without a request in context."""
        template = Template("{% load sitevars %}{% sitevar 'testvar' %}")
        rendered = template.render(Context({}))
        self.assertEqual(rendered.strip(), "testvalue")


class CheckContribSitesComesBeforeSitevarsTest(TestCase):
    @override_settings(
        INSTALLED_APPS=[
            "sitevars",
            "django.contrib.sites",
        ]
    )
    def test_sitevars_before_sites(self):
        result = checks.check_contrib_sites_comes_before_sitevars(None)
        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], Warning)
        self.assertEqual(result[0].id, "sitevars.W001")

    @override_settings(
        INSTALLED_APPS=[
            "django.contrib.sites",
            "sitevars",
        ]
    )
    def test_sites_before_sitevars(self):
        result = checks.check_contrib_sites_comes_before_sitevars(None)
        self.assertEqual(result, [])

    @override_settings(
        INSTALLED_APPS=[
            "django.contrib.sites",
        ]
    )
    def test_sites_only(self):
        # If we're not installed this should never run, but if it does, it should
        # produce no warnings.
        result = checks.check_contrib_sites_comes_before_sitevars(None)
        self.assertEqual(result, [])


class CheckRequestContextProcessorIsInstalledTest(TestCase):
    @override_settings(
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "OPTIONS": {
                    "context_processors": [
                        "django.template.context_processors.request",
                    ],
                },
            },
        ]
    )
    def test_context_processor_installed(self):
        """Test that no warning is returned when the context processor is installed."""
        result = checks.check_request_context_processor_is_installed(None)
        self.assertEqual(result, [])

    @override_settings(
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "OPTIONS": {
                    "context_processors": [],
                },
            },
        ]
    )
    def test_context_processor_not_installed(self):
        """Test that a warning is returned when the context processor is not installed."""
        result = checks.check_request_context_processor_is_installed(None)
        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], Warning)
        self.assertEqual(result[0].id, "sitevars.W002")



import logging
from django.db import models

logger = logging.getLogger("sitevars.testing")


class FakeSiteManager(models.Manager):
    def get_current(self, request):
        # Emulate the behavior of the Django sites framework
        logger.info("FakeSiteManager.get_current() called")
        return self.get(id=1)


class FakeSite(models.Model):
    name = models.CharField(max_length=100, blank=True)
    domain = models.CharField(max_length=100, blank=True)

    objects = FakeSiteManager()

    def __str__(self):
        return self.name

    @classmethod
    def get_current(cls, request):
        """For testing the CURRENT_SITE_METHOD setting."""
        logger.info("FakeSite.get_current() called")
        return cls.objects.get_current(request)


def get_current_site(request):
    """For testing the CURRENT_SITE_FUNCTION setting."""
    logger.info("get_current_site() called")
    return FakeSite.objects.get_current(request)
